#include "snake.h"
#include <stdlib.h>		/* getenv, ... */
#include <time.h>		/* time (for srand) */
#include <glob.h> 		/* glob_t etc. */
#include <string.h>		/* strcat, strcmp, ... */
#include <dlfcn.h>		/* dlopen, dlclose, ... */
#include <getopt.h>		/* getopt_long */

/* extern functions */
extern int init_graphics (void);
extern void draw_snakes (snake **);
extern void move_snakes (void);
extern void init_fruit (void);
extern int parse_input (void);
extern void draw_motion (void);
extern void print_delay (void);
extern void set_delay (int);
extern void open_config (void);
extern void close_config (void);
extern void init_modules (void);

/* private functions */
static void snake_play (void);
static void parse_cmdline (int, char **);
static void usage (char *);

/* public function */
int register_player(snake *s);
void finish (void);

/* public data */
snake *zoo[MAXPLAYERS];	/* snakes pointer */
int typein;             /* character typed in */
int delay = 40;		/* game delay in milliseconds */
char *rootdir;		/* installation root directory */
unsigned long int turns = 0;	/* game turns */
WINDOW *arenawin;


int main(int argc, char **argv)
{

	/* initialize the graphic or exit */
	if ( ! init_graphics() ) {
		fprintf(stderr, "can't initialize graphic");
		finish();
	}

	/* get installation root directory (default to ".") */
	setenv("SNAKEROOT", ".", 0);
	rootdir=getenv("SNAKEROOT");

	/* parse command line */
	parse_cmdline (argc, argv);

	/* report root directory value */
	fprintf(stderr, "SNAKEROOT=%s # set it yourself if needed\n", rootdir);

	/* initialize the random seed */
	srand(time(NULL));

	/* open config file */
	open_config();

	/* initialize the snakes */
	init_modules();

	/* close config file */
	close_config();

	/* initialize the fruit */
	init_fruit();
	
	/* draw the snakes */
	draw_snakes(zoo);

	/* refresh the screen */
	wrefresh(arenawin);

	for (;;)
	{
		/* parse keyboard input */
		if ( !parse_input() ) break;

		/* give players a turn to play */
		snake_play();

		/* move the snakes */
		move_snakes();

		/* draw the snakes */
		//draw_snakes(zoo);

		/* show activity */
		draw_motion();

		/* refresh the screen with the new data */
		wrefresh(arenawin);
		
		/* sleep for delay milliseconds */
		napms(delay);

		/* increment bogus time counter */
		turns++;

	}

	finish();
	exit(0); /* finish will exit itself */
}

void finish ()
{
	int i;

	/* reset the screen */
	endwin();

	/* print scores */
	for ( i=0; zoo[i]; i++ ) {
		printf("snake %i (%c): score(%li), length(%i)\n",
			i, (char)zoo[i]->look&A_CHARTEXT,
			zoo[i]->score, zoo[i]->length);
	}

	exit(0);
}

static void snake_play (void)
{
	int i;

	for (i=0; zoo[i]; i++)
	{
		if ( zoo[i]->died == 2 ) continue;
		(*zoo[i]->play)(zoo[i]);
	}
}


/* player modules must call this function at startup */
int register_player(snake *s)
{
	static int playernum=0;

	/* don't register more then MAXPLAYERS snakes ! */
	/* gotta remove this limitation            */
	if ( playernum >= MAXPLAYERS-1 ) {
		fprintf(stderr, "maximum count of players reached\n");
		return 0;
	}

	/* register new player */
	zoo[playernum++] = s;

	fprintf(stderr,  "player %d registered\n", playernum);
	return playernum-1;
}

/* get opts ... */
static void parse_cmdline (int argc, char **argv)
{
	static struct option opts[] = 
	{
		{ "delay", 1, 0, 'd' },
		{ "help", 0, 0, 'h' },
		{ "rootdir", 1, 0, 'r' },
		{ 0, 0, 0, 0 }
	};
	int opts_idx;
	int c;

	while (1)
	{

		/* scan next option argument */
 		c=getopt_long(argc, argv, "d:r:h", opts, &opts_idx);

		/* end of options */
		if ( c == -1 ) break;

		switch (c)
		{

	case 0:
		fprintf(stderr, "bug in getopt?\n");
		break;

	case 'd':
		set_delay(atoi(optarg));
		break;

	case 'r':
		rootdir = optarg;
		break;

	case 'h':
	case '?':
		usage(argv[0]);
		break; /* usage will exit anyway */

	default:
		fprintf(stderr, "getopt_long returned %c (?)\n", c);

		} /* end switch */

	} /* end while */
}

static void usage (char *me)
{
	fprintf(stderr, "Usage: %s [OPTIONS]\n", me);
	fprintf(stderr, "\n");
	fprintf(stderr, "  -d, --delay=time     set delay in millisecs\n");
	fprintf(stderr, "  -r, --rootdir=path   set SNAKEROOT\n");
	fprintf(stderr, "  -h, --help           show this screen and exit\n");
	exit(1);
}
